/**
 * This Mongo Redis Topic Consumer uses a mongo collection to persistenly represent a message stream.
 * Each document in the collection represents a message within the topic.
 *
 * A redis connection is used to broadcast signals when new messages arrive to unlock waiting consumers.
 * This allows us to avoid a polling mechanism inside the consumer code.
 */
import { Document } from "mongodb";
import { TopicConsumer } from "../../topicConsumer";
import { TopicMessageDocument, TopicMessageIdentifier } from "../../types";
export declare class MongoRedisTopicConsumer<T> extends TopicConsumer<T> {
    private mongoUrl;
    private databaseName;
    private collectionName;
    private redisUrl;
    private client;
    private redis;
    private isStarting;
    private isStreaming;
    private isPolling;
    private isMoreMessages;
    private isCrashed;
    private debuggingHandlers;
    private hasCalledNoMoreMessages;
    constructor(mongoUrl: string, databaseName: string, collectionName: string, redisUrl: string);
    start(): Promise<void>;
    stop(): Promise<void>;
    addDebuggingHandler(handler: (type: string, message: string) => void): void;
    private fireDebug;
    private getCollection;
    streamMessagesFrom(callback: (message: TopicMessageDocument<T>) => void, lastId?: null | TopicMessageIdentifier, queryDoc?: Document, noMoreMessages?: () => void): Promise<void>;
    private pollForNewMessages;
    private fetchNextMessagesFromMongo;
    private startListeningForNewMessageSignal;
}
