"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoClientController = void 0;
const mongodb_1 = require("mongodb");
// Creates and startsup a single MongoClient instance which is reusable
class MongoClientController {
    constructor(mongoUrl, connectionId) {
        this.mongoUrl = mongoUrl;
        this.connectionId = connectionId;
        this.client = null;
        this.isStarting = false;
        this.isStarted = false;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isStarting) {
                return false; // Already started up by something else
            }
            this.isStarting = true;
            const client = new mongodb_1.MongoClient(this.mongoUrl, {
                loggerLevel: "info",
                serverSelectionTimeoutMS: 0,
                connectTimeoutMS: 10 * 1000,
                socketTimeoutMS: 120 * 1000,
                keepAlive: true,
                keepAliveInitialDelay: 20 * 1000,
                heartbeatFrequencyMS: 20 * 1000,
                maxPoolSize: 1,
                /*
                directConnection: true, // This is important, it forces a single node style connection
                */
            });
            this.client = client;
            yield new Promise((resolve, reject) => {
                const timeoutSecs = 60;
                let isReady = false;
                client.addListener("error", (e) => {
                    console.error("MONGO CLIENT " + this.connectionId + ": ERROR:", e);
                });
                client.addListener("connectionCreated", (e) => {
                    // console.log("MONGO CLIENT " + this.connectionId + ": Connection created:", e);
                });
                client.addListener("connectionReady", (e) => {
                    // console.log("MONGO CLIENT " + this.connectionId + ": Connection is ready");
                });
                client.addListener("connectionClosed", (e) => {
                    console.error("MONGO CLIENT " + this.connectionId + ": Connection closed:", e);
                });
                client.addListener("serverClosed", (e) => {
                    console.error("MONGO CLIENT " + this.connectionId + ": Server closed:", e);
                });
                client.addListener("serverOpening", (e) => {
                    // console.log("MONGO CLIENT " + this.connectionId + ": Server opening:", e);
                });
                client.connect((err) => {
                    if (err) {
                        console.error('Error connecting to mongo: ' + err);
                        console.warn('Could not connect to mongo at: ' + this.mongoUrl);
                        reject(err);
                    }
                    else {
                        // console.log('Connected successfully to mongo');
                        resolve(null);
                    }
                });
                setTimeout(() => {
                    if (!isReady) {
                        reject(new Error("Timeout for mongo connection to become ready"));
                    }
                }, timeoutSecs * 1000);
            });
            this.isStarted = true;
        });
    }
    getClient() {
        if (this.client === null) {
            throw new Error("You must startup this controller");
        }
        else {
            if (this.isStarted) {
                return this.client;
            }
            else {
                throw new Error("Mongo client still starting up, please await the startup function properly");
            }
        }
    }
}
exports.MongoClientController = MongoClientController;
