"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisClientController = void 0;
const ioredis_1 = __importDefault(require("ioredis"));
class RedisClientController {
    constructor(redisUrl, connectionId) {
        this.redisUrl = redisUrl;
        this.connectionId = connectionId;
        this.client = null;
        this.isStarting = false;
        this.isStarted = false;
        this.eventsCallback = () => { };
    }
    getNewRedisClient() {
        const client = new ioredis_1.default(this.redisUrl, {
            lazyConnect: true,
            autoResubscribe: false,
            commandQueue: false,
            enableOfflineQueue: false,
            retryStrategy(times) {
                const delay = Math.min(times * 500, 5000);
                return delay;
            },
        });
        // console.log("Creating redis client: " + this.connectionId + "_" + name);
        return client;
    }
    start(eventsCallback) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isStarting) {
                return false; // Already started up by something else
            }
            this.isStarting = true;
            this.eventsCallback = eventsCallback;
            this.client = this.getNewRedisClient();
            this.client.on('error', (err) => {
                console.error("REDIS CLIENT " + this.connectionId + ": Error", err);
            });
            this.client.on('connect', () => {
                // console.log("REDIS CLIENT " + this.connectionId + ": Connected");
            });
            this.client.on('ready', () => {
                // console.log("REDIS CLIENT " + this.connectionId + ": Ready");
                if (this.eventsCallback !== null) {
                    this.eventsCallback("ready", this.connectionId);
                }
            });
            this.client.on('close', () => {
                console.error("REDIS CLIENT " + this.connectionId + ": Close");
            });
            this.client.on('reconnecting', () => {
                console.warn("REDIS CLIENT " + this.connectionId + ": Reconnecting...");
                if (this.eventsCallback !== null) {
                    this.eventsCallback("reconnecting", this.connectionId);
                }
            });
            yield this.client.connect();
            this.isStarted = true;
        });
    }
    getClient() {
        if (this.client === null) {
            throw new Error("You must startup this controller");
        }
        else {
            if (this.isStarted) {
                return this.client;
            }
            else {
                throw new Error("Redis client still starting up, please await the startup function properly");
            }
        }
    }
}
exports.RedisClientController = RedisClientController;
